<?php

namespace Drupal\library_manager\Tests;

/**
 * Tests the Library information interface.
 *
 * @group library_manager
 */
class LibraryInformationTest extends TestBase {

  /**
   * Tests collection page.
   */
  public function testCollectionPage() {

    $this->drupalGet('admin/structure/library');

    $this->assertPageTitle('Libraries');

    $this->assertByXpath('//input[@data-drupal-selector="library-filter" and @placeholder="Library name or extension"]');

    $expected_header = [
      'Name',
      'Extension',
      'Version',
      'License',
      'Operations',
    ];

    $this->assertTableHeader('//main//table//th', $expected_header);

    $found_libraries = [];
    foreach ($this->xpath('//main//table/tbody/tr') as $tr) {
      // Do not check library version because it may change.
      $found_libraries[(string) $tr->td[0]] = [
        'extension' => (string) $tr->td[1],
        'license' => (string) $tr->td[3],
      ];
    }

    // Check some arbitrary libraries.
    $this->assertEqual(
      $found_libraries['backbone'],
      [
        'extension' => 'core',
        'license' => 'MIT',
      ],
      'Backbone library was found.'
    );

    $this->assertEqual(
      $found_libraries['drupal.system.modules'],
      [
        'extension' => 'system',
        'license' => 'GNU-GPL-2.0-or-later',
      ],
      'System modules library was found.'
    );

    $this->assertEqual(
      $found_libraries['library_manager'],
      [
        'extension' => 'library_manager',
        'license' => 'GNU-GPL-2.0-or-later',
      ],
      'Library manager library was found.'
    );
  }

  /**
   * Tests details page.
   */
  public function testDetailsPage() {

    $this->drupalGet('admin/structure/library');
    $details_link = $this->xpath('//main//table//td[position() = 1 and text() = "library_manager_test"]/following-sibling::td[position() = 4 ]//a[text() = "View"]')[0];
    $this->drupalGet($this->getAbsoluteUrl($details_link['href']));

    $this->assertPageTitle('library_manager_test');

    $items = $this->xpath('//div[contains(@class, "form-type-item")]');

    $data = [];
    foreach ($items as $item) {
      $data[(string) $item->label] = trim($item);
    }

    $expected_data = [
      'ID' => 'library_manager_test',
      'Version' => '1.2.3',
      'License' => 'GNU-GPL-2.0-or-later',
    ];

    $this->assertEqual($data, $expected_data);

    $fieldsets = $this->xpath('//fieldset[contains(@class, "form-wrapper")]');

    // JS files.
    $this->assertEqual($fieldsets[0]->legend->span, 'JS', 'JS label was found.');
    $this->assertEqual($fieldsets[0]->div->ul->li->a, 'modules/library_manager/tests/library_manager_test/js/example.js', 'JS value was found.');

    // CSS files.
    $this->assertEqual($fieldsets[1]->legend->span, 'CSS', 'CSS label was found.');
    $this->assertEqual($fieldsets[1]->div->ul->li->a, 'modules/library_manager/tests/library_manager_test/css/example.css', 'CSS value was found.');

    // Dependencies.
    $this->assertEqual($fieldsets[2]->legend->span, 'Dependencies', 'Dependencies label was found.');
    $this->assertEqual($fieldsets[2]->div->ul->li[0]->a, 'core/jquery');
    $this->assertEqual($fieldsets[2]->div->ul->li[1]->a, 'core/drupal');
    $this->assertEqual($fieldsets[2]->div->ul->li[2]->a, 'core/drupal.dialog.ajax');

    // Check if dependency link is valid.
    $this->clickLink('core/jquery');
    $this->assertPageTitle('jquery');
  }

  /**
   * Tests details page.
   */
  public function testAutocompletePage() {
    $this->drupalGet('admin/structure/library/autocomplete', ['query' => ['q' => 'backbone']]);
    $this->assertResponse(200);
    $this->assertHeader('Content-Type', 'application/json');
    $data = json_decode($this->getRawContent());
    $this->assertEqual($data[0]->label, 'core/backbone');
    $this->assertEqual($data[0]->value, 'core/backbone');
  }

}
